using System;
using System.Collections;
using Server.Gumps;
using Server.Multis;
using Server.Network;
using Server.Engines.Plants;
using System.Collections.Generic;
using Server.ContextMenus;
using Server.Prompts;
using Server.Targeting;

namespace Server.Items
{
    public class LapisStatueDyeTub : LapisAbstractDyeTub
    {

        [Constructable]
        public LapisStatueDyeTub()
        {
            this.Name = "AA߉[Statue]";
            this.LootType = LootType.Blessed;
            this.nCategory = 3;
        }

        public LapisStatueDyeTub(Serial serial)
            : base(serial)
        {
        }

        private int nCategory = 0;
        private bool[] colorUsable = new bool[] { false,false,false,false};

        private bool normalColor  = false;
        private bool specialColor = false;
        private bool ingotsColor  = false;
        private bool neonColor    = true;

        public override int Category { get { return nCategory; } set { this.nCategory = value; this.InvalidateProperties(); } }
        public override bool ColorNormal { get { return normalColor; } set { this.normalColor = value; } }
        public override bool ColorSpecial { get { return specialColor; } set { this.specialColor = value; } }
        public override bool ColorIngots { get { return ingotsColor; } set { this.ingotsColor = value; } }
        public override bool ColorNeon { get { return neonColor; } set { this.neonColor = value; } }

        public override bool AllowDyables
        {
            get
            {
                return false;
            }
        }
        public override bool AllowStatuettes
        {
            get
            {
                return true;
            }
        }
        public override int TargetMessage
        {
            get
            {
                return 1049777;
            }
        }
        public override int FailMessage
        {
            get
            {
                return 1049778;
            }
        }
        public override int LabelNumber
        {
            get
            {
                return 1049741;
            }
        }
        public override CustomHuePicker CustomHuePicker
        {
            get
            {
                return LapisAbstractDyeTub.HuePicker[this.Category];
            }
        }
        public override void OnDoubleClick(Mobile from)
        {
 
            base.OnDoubleClick(from);
        }

        public override void GetProperties(ObjectPropertyList list)
        {
            base.GetProperties(list);
            list.Add(1071518, LapisAbstractDyeTub.DispName[this.Category]);
        }

        public override void GetContextMenuEntries(Mobile from, List<ContextMenuEntry> list)
        {
            base.GetContextMenuEntries(from, list);

            if (from.CheckAlive())
                list.Add(new LapisChangeColorEntry(from, this));

            SetSecureLevelEntry.AddTo(from, this, list);
        }

        public override void SetGumpEntry(Mobile mob)
        {
            bool[] wkbool = new bool[] { this.ColorNormal, this.ColorSpecial, this.ColorIngots, this.ColorNeon };
            mob.SendGump(new LapisChangeColorGump((LapisAbstractDyeTub)this ,wkbool, this.nCategory));
        }

        public override void selectPaints(Mobile from)
        {
            if (from.InRange(GetWorldLocation(), 1))
            {
                from.SendMessage("yLIĂB");
                from.Target = new InternalTarget(this);
                this.InvalidateProperties();
            }
            else
            {
                from.SendLocalizedMessage(500446); // That is too far away.
            }
        }
        private class InternalTarget : Target
        {
            private readonly LapisAbstractDyeTub mItem;

            public InternalTarget(LapisAbstractDyeTub item)
                : base(1, false, TargetFlags.None)
            {
                mItem = item;
            }

            protected override void OnTarget(Mobile from, object targeted)
            {
                if (targeted is Item)
                {
                    Item item = (Item)targeted;
                    if (item is LapisChangeColorPaints)
                    {
                        LapisChangeColorPaints cp = (LapisChangeColorPaints)item;
                        // łɒǉĂFłΒe
                        if ( (mItem.ColorNormal  && cp.nCategory == 0) ||
                             (mItem.ColorSpecial && cp.nCategory == 1) ||
                             (mItem.ColorIngots  && cp.nCategory == 2) ||
                             (mItem.ColorNeon    && cp.nCategory == 3))
                        {
                            from.SendMessage("łɒǉĂyLłB");
                            return;
                        }
                        switch  (cp.nCategory)
                        {
                            case 0:
                                mItem.ColorNormal = true;
                                break;
                            case 1:
                                mItem.ColorSpecial = true;
                                break;
                            case 2:
                                mItem.ColorIngots = true;
                                break;
                            case 3:
                                mItem.ColorNeon = true;
                                break;
                        }
                        from.SendMessage("yLǉ܂I");
                        cp.Delete();
                    }
                    else
                    {
                        from.SendMessage("ɂ͒ǉł܂B");
                        return;
                    }
                }
                else
                {
                    from.SendMessage("ɂ͒ǉł܂B");
                    return;
                }
            }
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);

            writer.Write((int)0); // version
            writer.Write(this.nCategory);
            writer.Write(this.ColorNormal);
            writer.Write(this.ColorSpecial);
            writer.Write(this.ColorIngots);
            writer.Write(this.ColorNeon);
        }

        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);

            int version = reader.ReadInt();
            this.Category = reader.ReadInt();
            this.ColorNormal = reader.ReadBool();
            this.ColorSpecial = reader.ReadBool();
            this.ColorIngots = reader.ReadBool();
            this.ColorNeon = reader.ReadBool();
        }
    }
}